"use strict";

const EXPORTED_SYMBOLS = ["XB"];

const { classes: Cc, interfaces: Ci, utils: Cu, results: Cr } = Components;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");

var sysutils,
    fileutils,
    strutils,
    xmlutils,
    misc,
    CustomErrors,
    Base,
    BarPlatform;

const XB = {
    init: function XB_init(application) {
        this._application = application;
        BarPlatform = application.BarPlatform;
        let (appCoreLib = application.core.Lib) {
            sysutils = appCoreLib.sysutils;
            fileutils = appCoreLib.fileutils;
            strutils = appCoreLib.strutils;
            xmlutils = appCoreLib.xmlutils;
            misc = appCoreLib.misc;
            CustomErrors = appCoreLib.CustomErrors;
            Base = appCoreLib.Base;
        };
        
        this._logger = application.getLogger("XB");
        this._loadModules();
        
        XB._base.init(application);
        XB._Parser.init();
        XB.cache.init();
        XB.network.init();
        
        application.core.xbProtocol.setDataProvider("toolkit", this._base.toolkitDataProvider);
    },
    
    finalize: function XB_finalize(doCleanup) {
        this._application.core.xbProtocol.setDataProvider("toolkit", null);
        XB.network.finalize();
        XB.cache.finalize(doCleanup);
        XB._base.finalize();
    },
    
    _application: null,
    
    _modules: [
        "xbbase.js", "xbtypes.js", "xbcalcnodes.js", "xbparser.js", "xbwidget.js",
        "xbcache.js", "xbnetwork.js", "xbfuncs.js"
    ],
    
    _loadModules: function XB__loadModules() {
        const mozSSLoader = Cc["@mozilla.org/moz/jssubscript-loader;1"].getService(Ci.mozIJSSubScriptLoader);
        const xbDirPath = this._application.partsURL + "xb/";
        this._modules.forEach(function xb_loadModule(moduleFileName) {
            this._logger.debug("  Loading module " + moduleFileName);
            mozSSLoader.loadSubScript(xbDirPath + moduleFileName);
        }, this);
    }
};
