"use strict";

const {
    classes: Cc,
    interfaces: Ci,
    results: Cr,
    utils: Cu
} = Components;
const EXTENSION_PATH = Cc["@mozilla.org/network/io-service;1"]
                           .getService(Ci.nsIIOService)
                           .newFileURI(__LOCATION__.parent.parent)
                           .spec;

Cc["@mozilla.org/moz/jssubscript-loader;1"]
    .getService(Ci.mozIJSSubScriptLoader)
    .loadSubScript(EXTENSION_PATH + "config.js");

Cu.import("resource://gre/modules/XPCOMUtils.jsm");


const PROTOCOL_SCHEME = "bar";
const IOS = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);

var gHandlers = [];

function BarProtocolHandler() {}

BarProtocolHandler.prototype = {
    get wrappedJSObject() {
        return WrappedObject;
    },
    
    get scheme() {
        return PROTOCOL_SCHEME;
    },
    
    get protocolFlags() {
        return Ci.nsIProtocolHandler.URI_DANGEROUS_TO_LOAD;
    },
    
    get defaultPort() {
        return -1;
    },
    
    allowPort: function BPH_allowPort(aPort, aScheme) {
        return false;
    },
    
    newURI: function BPH_newURI(aSpec, aOriginalCharset, aBaseURI) {
        let uri = this._newURI(aSpec, aOriginalCharset, aBaseURI);
        
        if (!uri)
            throw Cr.NS_ERROR_MALFORMED_URI;
        
        return uri;
    },
    
    newChannel: function BPH_newChannel(aURI) {
        let simpleSpec = aURI.spec.replace(this.scheme + ":", "");
        let channel = this._newChannel(aURI, simpleSpec);
        
        if (!channel) {
            channel = IOS.newChannel("data:,", null, null);
            channel.originalURI = aURI;
        }
        
        return channel;
    },
    
    _newURI: function BPH__newURI(aSpec, aOriginalCharset, aBaseURI) {
        let simpleSpec = aSpec.replace(this.scheme + ":", "");
        let baseURISpec = ((aBaseURI && aBaseURI.spec) || "").replace(this.scheme + ":", "");
        
        let uri = null;
        gHandlers.some(function(handler) {
            return null !== (uri = handler.newURI(aSpec, aOriginalCharset, aBaseURI, simpleSpec, baseURISpec));
        });
        
        if (uri && typeof uri == "string")
            uri = IOS.newURI(uri, aOriginalCharset, aBaseURI);
        
        return uri;
    },
    
    _newChannel: function BPH__newChannel(aURI, aSimpleSpec) {
        let channel;
        gHandlers.some(function(handler) null !== (channel = handler.newChannel(aURI, aSimpleSpec)));
        return channel || null;
    },
    
    classDescription: "Custom Yandex bar protocol handler",
    classID: Components.ID("f597d51a-466e-11e0-9f38-6310e2061bfe"),
    contractID: ("@mozilla.org/network/protocol;1?name=" + PROTOCOL_SCHEME),
    QueryInterface: XPCOMUtils.generateQI([Ci.nsIProtocolHandler]),
    
    _xpcom_categories: [{
        category: "app-startup",
        service: true
    }]
};

var WrappedObject = {
    get scheme() {
        return PROTOCOL_SCHEME;
    },
    
    addHandler: function WrappedObject_addHandler(aHandler) {
        if (!gHandlers.some(function(handler) aHandler === handler))
            gHandlers.push(aHandler);
    },
    
    removeHandler: function WrappedObject_removeHandler(aHandler) {
        gHandlers = gHandlers.filter(function(handler) aHandler !== handler);
    }
};

if (XPCOMUtils.generateNSGetFactory)
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([BarProtocolHandler]);
else
    var NSGetModule = XPCOMUtils.generateNSGetModule([BarProtocolHandler]);
